@@ -17,7 +17,7 @@
 # along with Clusterix; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-# Clusterixwsd version 4.8
+# Clusterixwsd version 5.1
 # 0. The cluster write information in local quorum file and a timestamp
 # in the quorum file of other node
 # 1. Put date and node in the log informations
@@ -79,6 +79,11 @@
 # 44. Added force crash of machine if the umount dont function
 # 45. Added possibility to make fsck before to mount device
 # 46. Changed the name of interface variable in configuration file
+# 47. Introduce locks for start and stop sequence and change script.sh
+# to mantain lock during multiple start stop sequence
+# 48. Changed send arp procedure to permit virtual ip on more interfaces
+# 49. Changed procedure to obtain macaddress.now macaddress is obtained
+# dinamically by the interface setting.
 
 # Configuration file
 conffile="/opt/clusterix/clusterixwsd.conf"
@@ -118,12 +123,13 @@ count="$1"
 interface="$(eval $echo \$interface${count})"
 interfacenumber="$(eval $echo \$interfacenumber${count})"
 vip="$(eval $echo \$vip${count})"
-netmask="$(eval $echo \$netmask${count})"
-broadcast="$(eval $echo \$broadcast${count})"
+netmask="$($ifconfig $interface | $grep inet| $awk -F: '{print $4}')"
+broadcast="$($ifconfig $interface | $grep inet| $awk  '{print $3}'| $awk -F: '{print $2}')"
+iammacaddress="$($ifconfig $interface | $grep $interface | $awk '{print $5}')"
 $echo "$date $iam: Configuring virtual ip address $vip ..."
 $ifconfig $interface:$interfacenumber $vip netmask $netmask broadcast $broadcast up
-$echo "$date $iam: Publishing macaddress $iammacaddress for $iam ..."
-$send_arp $vip $(eval $echo \$${iam}macaddress) $broadcast ffffffffffff
+$echo "$date $iam: Publishing macaddress $iammacaddress for interface $interface for $iam ..."
+$send_arp $vip $iammacaddress $broadcast ffffffffffff $interface
 }
 
 # unset virtual ip address
@@ -232,7 +238,7 @@ netmask="$(eval $echo \$netmask${count})"
 broadcast="$(eval $echo \$broadcast${count})"
 $echo "$date $iam: Configuring virtual ip address $vip ..."
 $ifconfig $interface:$interfacenumber plumb
-$ifconfig $interface:$interfacenumber $vip netmask $netmask broadcast $broadcast up
+$ifconfig $interface:$interfacenumber $vip up
 $echo "$date $iam: Publishing macaddress $iammacaddress for $iam ..."
 }
 
@@ -316,8 +322,6 @@ eval ${node1}ipprivatenetwork2="$node1ipprivatenetwork2"
 eval ${node2}ipprivatenetwork2="$node2ipprivatenetwork2"
 eval ${node1}ippublicnetwork="$node1ippublicnetwork"
 eval ${node2}ippublicnetwork="$node2ippublicnetwork"
-eval ${node1}macaddress="$node1macaddress"
-eval ${node2}macaddress="$node2macaddress"
 }
 
 # Variable definition to start virtual ip address
@@ -798,6 +802,28 @@ $echo "Initialization Date of this node ($node): 		\"$initializationdatenode\""
 $echo ""
 }
 
+printlocks () {
+$echo "Cluster Locks Information for node ### $node ### :"
+$echo ""
+$echo "Start Locks: $lockstartservicebefore , $lockstartserviceafter"
+$echo "Stop Locks:  $lockstopservice"
+$echo ""
+}
+
+readlocks () {
+if [ "$1" = "iam" ]
+then
+lockstartservicebefore=$($dd bs=$blocksize count=1 skip=$(eval $echo \$${1}blockstartservicebefore) conv=sync,notrunc if=$quorumfile)
+lockstartserviceafter=$($dd bs=$blocksize count=1 skip=$(eval $echo \$${1}blockstartserviceafter) conv=sync,notrunc if=$quorumfile)
+lockstopservice=$($dd bs=$blocksize count=1 skip=$(eval $echo \$${1}blockstopservice) conv=sync,notrunc if=$quorumfile)
+
+else
+lockstartservicebefore=$($ssh $networktouse $dd bs=$blocksize count=1 skip=$(eval $echo \$${1}blockstartservicebefore) conv=sync,notrunc if=$quorumfile)
+lockstartserviceafter=$($ssh $networktouse $dd bs=$blocksize count=1 skip=$(eval $echo \$${1}blockstartserviceafter) conv=sync,notrunc if=$quorumfile)
+lockstopservice=$($ssh $networktouse $dd bs=$blocksize count=1 skip=$(eval $echo \$${1}blockstopservice) conv=sync,notrunc if=$quorumfile)
+
+fi
+}
 
 # Chech if a node is initialized.
 checkinitialization () {
@@ -811,6 +837,51 @@ $echo "Node Cluster  $iam not initialized."
 fi
 }
 
+checklockstartlocal () {
+iamlockstartservicebefore=$($dd bs=$blocksize count=1 skip=$iamblockstartservicebefore conv=sync,notrunc if=$quorumfile)
+iamlockstartserviceafter=$($dd bs=$blocksize count=1 skip=$iamblockstartserviceafter conv=sync,notrunc if=$quorumfile)
+iamlockstopservice=$($dd bs=$blocksize count=1 skip=$iamblockstopservice conv=sync,notrunc if=$quorumfile)
+if [ "$iamlockstartservicebefore" = "FREE" ] && [ "$iamlockstartserviceafter" = "FREE" ] && [ "$iamlockstopservice" = "FREE" ]
+then
+$echo "$date $iam: no start locks found on $iam. go on..."
+else
+$echo "$date $iam: Start locks found on $iam. Start sequence aborted."
+exit 1
+fi
+}
+
+checklockstartremote () {
+$chooseinterface
+if [ $networktouse != "none" ]
+then
+youarelockstartservicebefore=$($ssh $networktouse $dd bs=$blocksize count=1 skip=$youareblockstartservicebefore conv=sync,notrunc if=$quorumfile)
+youarelockstartserviceafter=$($ssh $networktouse $dd bs=$blocksize count=1 skip=$youareblockstartserviceafter conv=sync,notrunc if=$quorumfile)
+youarelockstopservice=$($ssh $networktouse $dd bs=$blocksize count=1 skip=$youareblockstopservice conv=sync,notrunc if=$quorumfile)
+if [ "$youarelockstartservicebefore" = "FREE" ] && [ "$youarelockstartserviceafter" = "FREE" ] && [ "$youarelockstopservice" = "FREE" ]
+then
+$echo "$date $iam: no start locks found on $youare. go on..."
+else
+$echo "$date $iam: Start locks found on $youare. Start sequence aborted."
+exit 1
+fi
+else
+$echo "$date $iam: All avalaible network are down: Cannot check for locks on $youare.go on..."
+fi
+}
+
+checklockstop () {
+iamlockstartserviceafter=$($dd bs=$blocksize count=1 skip=$iamblockstartserviceafter conv=sync,notrunc if=$quorumfile)
+iamlockstopservice=$($dd bs=$blocksize count=1 skip=$iamblockstopservice conv=sync,notrunc if=$quorumfile)
+if [ "$iamlockstartserviceafter" = "FREE" ] && [ "$iamlockstopservice" = "FREE" ]
+then
+$echo "$date $iam: no stop locks found on $iam. go on..."
+else
+$echo "$date $iam: Stop locks found on $iam. Start sequence aborted."
+exit 1
+fi
+}
+
+
 # Start of the service
 startservice () {
 if [ $iamactive = "no" ]
@@ -822,9 +893,14 @@ fi
 
 # Force start service without checking who is active
 startserviceforce () {
-$echo "$date $iam: Beginning start sequence..."
+checklockstartlocal
+checklockstartremote
+$echo "$date $iam: Setting startbeforestop lock..."
+writestartlocalbefore "LOCKED" 2> /dev/null
 $echo "$date $iam: Resetting all services..."
 $pathcluster stopservice
+$echo "$date $iam: Setting startafterstop lock..."
+writestartlocalafter "LOCKED" 2> /dev/null
 $echo "$date $iam: Launching stop services on other node ($youare) ..."
 remotestop &
 killremotestop
@@ -835,11 +911,18 @@ mailsend
 $script "$servicename" "$log" "$start_service_comand" "$stop_service_comand" "$includestringstart" "$excludestringstart" "$includestringstop" "$excludestringstop" "$timeoutstop" "$timeoutstart" "$ps" "$psoption" "$mailto" "$iam" "$countstartlimit" "$countstart" "$begincheck" "$stop_service_comand_2" "$timeoutstop_2" start
 iamactive="yes"
 $echo "$date" | $dd bs=$blocksize count=1 seek=$(eval $echo $iamblockstartserviceforcedate) conv=sync,notrunc of=$quorumfile
+$echo "$date $iam: Unsetting startafterstop lock..."
+writestartlocalafter "FREE" 2> /dev/null
+$echo "$date $iam: Unsetting startbeforestop lock..."
+writestartlocalbefore "FREE" 2> /dev/null
 }
 
 # Force stop service
 stopserviceforce () {
+checklockstop 2> /dev/null
 $pathcluster stopcontrol
+$echo "$date $iam: Setting stop lock..."
+writestoplocal "LOCKED" 2> /dev/null
 $echo "$date $iam: Stopping service on node $iam..."
 $script "$servicename" "$log" "$start_service_comand" "$stop_service_comand" "$includestringstart" "$excludestringstart" "$includestringstop" "$excludestringstop" "$timeoutstop" "$timeoutstart" "$ps" "$psoption" "$mailto" "$iam" "$countstartlimit" "$countstart" "$begincheck" "$stop_service_comand_2" "$timeoutstop_2" stop
 killcommand
@@ -851,6 +934,8 @@ crashnode
 fi
 stopvip
 iamactive="no"
+$echo "$date $iam: Unsetting stop lock..."
+writestoplocal "FREE" 2> /dev/null
 }
 
 # Stop service
@@ -942,6 +1027,9 @@ eval ${1}blockinitializationdate=7
 eval ${1}blockdatenumothernode=8
 eval ${1}blockactiveothernode=9
 eval ${1}blockstartserviceforcedate=10
+eval ${1}blockstartservicebefore=11
+eval ${1}blockstartserviceafter=12
+eval ${1}blockstopservice=13
 
 # Information taken by node2
 eval ${2}blocknode=20
@@ -955,6 +1043,9 @@ eval ${2}blockinitializationdate=27
 eval ${2}blockdatenumothernode=28
 eval ${2}blockactiveothernode=29
 eval ${2}blockstartserviceforcedate=30
+eval ${2}blockstartservicebefore=31
+eval ${2}blockstartserviceafter=32
+eval ${2}blockstopservice=33
 }
 
 #  Read information from quorum file
@@ -1034,6 +1125,12 @@ $ssh $networktouse "$echo "Initializing.." | $dd bs=$blocksize count=1 seek=$(ev
 $echo "Block $(eval $echo \$${1}blockactiveothernode) initialized on node $youare in $quorumfile."
 $echo "$date" | $dd bs=$blocksize count=1 seek=$(eval $echo \$${1}blockstartserviceforcedate) conv=sync,notrunc of=$quorumfile
 $echo "Block $(eval $echo \$${1}blockstartserviceforcedate) initialized on node ${!1} in $quorumfile."
+$echo "FREE" | $dd bs=$blocksize count=1 seek=$(eval $echo \$${1}blockstartservicebefore) conv=sync,notrunc of=$quorumfile
+$echo "Block $(eval $echo \$${1}blockstartservicebefore) initialized on node ${!1} in $quorumfile."
+$echo "FREE" | $dd bs=$blocksize count=1 seek=$(eval $echo \$${1}blockstartserviceafter) conv=sync,notrunc of=$quorumfile
+$echo "Block $(eval $echo \$${1}blockstartserviceafter) initialized on node ${!1} in $quorumfile."
+$echo "FREE" | $dd bs=$blocksize count=1 seek=$(eval $echo \$${1}blockstopservice) conv=sync,notrunc of=$quorumfile
+$echo "Block $(eval $echo \$${1}blockstopservice) initialized on node ${!1} in $quorumfile."
 $echo "$date $iam: Node Cluster ${!1} is initialized at $date ."
 else
 $ssh $networktouse "$echo "${!1}" | $dd bs=$blocksize count=1 seek=$(eval $echo \$${1}blocknode) conv=sync,notrunc of=$quorumfile"
@@ -1058,6 +1155,12 @@ $echo "Initializing.." | $dd bs=$blocksize count=1 seek=$(eval $echo \$${1}block
 $echo "Block $(eval $echo \$${1}blockactiveothernode) initialized on node $iam in $quorumfile."
 $ssh $networktouse "$echo "$date" | $dd bs=$blocksize count=1 seek=$(eval $echo \$${1}blockstartserviceforcedate) conv=sync,notrunc of=$quorumfile"
 $echo "Block $(eval $echo \$${1}blockstartserviceforcedate) initialized on node ${!1} in $quorumfile."
+$ssh $networktouse "$echo "FREE" | $dd bs=$blocksize count=1 seek=$(eval $echo \$${1}blockstartservicebefore) conv=sync,notrunc of=$quorumfile"
+$echo "Block $(eval $echo \$${1}blockstartservicebefore) initialized on node ${!1} in $quorumfile."
+$ssh $networktouse "$echo "FREE" | $dd bs=$blocksize count=1 seek=$(eval $echo \$${1}blockstartserviceafter) conv=sync,notrunc of=$quorumfile"
+$echo "Block $(eval $echo \$${1}blockstartserviceafter) initialized on node ${!1} in $quorumfile."
+$ssh $networktouse "$echo "FREE" | $dd bs=$blocksize count=1 seek=$(eval $echo \$${1}blockstopservice) conv=sync,notrunc of=$quorumfile"
+$echo "Block $(eval $echo \$${1}blockstopservice) initialized on node ${!1} in $quorumfile."
 $echo "$date $iam: Node Cluster ${!1} is initialized at $date ."
 fi
 fi
@@ -1085,6 +1188,19 @@ $echo "Block $(eval $echo \$iamblockactive) initialized on node ${iam} in $quoru
 fi
 }
 
+writestartlocalbefore () {
+$echo "$1" | $dd bs=$blocksize count=1 seek=$(eval $echo \$iamblockstartservicebefore) conv=sync,notrunc of=$quorumfile
+}
+
+writestartlocalafter () {
+$echo "$1" | $dd bs=$blocksize count=1 seek=$(eval $echo \$iamblockstartserviceafter) conv=sync,notrunc of=$quorumfile
+}
+
+writestoplocal () {
+$echo "$1" | $dd bs=$blocksize count=1 seek=$(eval $echo \$iamblockstopservice) conv=sync,notrunc of=$quorumfile
+}
+
+
 # Routine that control if the other node is up and writing on quorum device
 controlquorum () {
 datemom=$youaredatenum
@@ -1225,17 +1341,41 @@ stopcluster
 status)
 readinf iam 1> $tty 2> /dev/null
 printinf 1> $tty 2> /dev/null
+readlocks iam 1> $tty 2> /dev/null
+printlocks 1> $tty 2> /dev/null
 $chooseinterface
 if [ $networktouse != "none" ]
 then
 readinf youare 1> $tty 2> /dev/null
 printinf 1> $tty 2> /dev/null
+readlocks youare 1> $tty 2> /dev/null
+printlocks 1> $tty 2> /dev/null
 else
 echo $echooption "${statusko}ERROR${statusclose}: All avalaible network are down:  cannot take information about $youare status." 1> $tty 2> /dev/null
 fi
 chooseinterfacestatus
 ;;
 
+clearlocks)
+$echo "$date $iam: Unsetting startafterstop lock..." 1> $tty 2> /dev/null
+writestartlocalafter "FREE"
+$echo "$date $iam: Unsetting startbeforestop lock..." 1> $tty 2> /dev/null
+writestartlocalbefore "FREE"
+$echo "$date $iam: Unsetting stop lock..." 1> $tty 2> /dev/null
+writestoplocal "FREE"
+
+;;
+
+setlocks)
+$echo "$date $iam: Setting startafterstop lock..." 1> $tty 2> /dev/null
+writestartlocalafter "LOCKED"
+$echo "$date $iam: Setting startbeforestop lock..." 1> $tty 2> /dev/null
+writestartlocalbefore "LOCKED"
+$echo "$date $iam: Setting stop lock..." 1> $tty 2> /dev/null
+writestoplocal "LOCKED"
+
+;;
+
 initialize)
 initialization iam 1> $tty 2> /dev/null
 initialization youare 1> $tty 2> /dev/null
@@ -1323,10 +1463,10 @@ writeactive $2
 version) $echo "$version"  1> $tty 
 ;;
 remote)
-if [ $2 != "start" ] && [ $2 != "stop" ] && [ $2 != "startservice" ] && [ $2 != "stopservice" ]
+if [ $2 != "start" ] && [ $2 != "stop" ] && [ $2 != "startservice" ] && [ $2 != "stopservice" ] && [ $2 != "clearlocks" ] && [ $2 != "setlocks" ]
 then
 $echo "$version..." 1> $tty
-$echo "Usage: $pathcluster remote {start|stop|startservice|stopservice}" 1> $tty
+$echo "Usage: $pathcluster remote {start|stop|startservice|stopservice|clearlocks|setlocks}" 1> $tty
 $echo "" 1> $tty
 else
 remote $2
@@ -1335,7 +1475,7 @@ fi
 
 *)
 	$echo "$version..." 1> $tty
-        $echo "Usage: $pathcluster {start|stop|startforeground|startall|stopall|status|initialize|startservice|stopservice|stopcluster|stopclusterhere|startcontrol|stopcontrol|statuscontrol|writedatenow|writeactive|version}" 1> $tty
+        $echo "Usage: $pathcluster {start|stop|startforeground|startall|stopall|status|initialize|startservice|stopservice|stopcluster|stopclusterhere|startcontrol|stopcontrol|statuscontrol|writedatenow|writeactive||clearlocks|version}" 1> $tty
 	$echo "" 1> $tty
 	$echo "start:	 		start cluster in background: service and check quorum device." 1> $tty
 	$echo "stop: 			stop cluster: service and check quorum device." 1> $tty
@@ -1352,9 +1492,10 @@ fi
 	$echo "stopservice: 		stop only the service not the check of quorum device." 1> $tty
 	$echo "stopall: 		stop service,stop check quorum device and stop processes that control the availibility of service." 1> $tty
 	$echo "startall: 		start service,start check quorum device and start processes that control the availibility of service. Use this only if the cluster is down also in the other node.Otherwise use $pathcluster start." 1> $tty
-	$echo "remote {start|stop|startservice|stopservice}: start,stop,startservice,stopservice on the other node." 1> $tty
+	$echo "remote {start|stop|startservice|stopservice|clearlocks}: start,stop,startservice,stopservice,clearlocks on the other node." 1> $tty
 	$echo "writeactive {yes|no}: Write yes,no for the status on quorum device." 1> $tty
 	$echo "writedatenow: Write actual date on quorum device." 1> $tty
+	$echo "clearlocks: Set FREE status on all the locks." 1> $tty
 	$echo "version: 		Program version." 1> $tty
 	        exit 1
 
