@@ -23,7 +23,7 @@ sh /opt/clusterix/gotest.sh &
 ;;
 stop)
 pid=$(ps auxw | grep gotest | grep -v grep | awk '{print $2}')
-if [ ! -z $pid ]
+if [ ! -z "$pid" ]
 then
 kill -9 $pid
 fi
